/**
 * @file web/controller.js
 *
 * Provides element appending, insertion, deletion with content
 * provided by the server.
 */

/**
 * Logs error while fetching from server.
 *
 * @param {string} url resource to fetch
 * @param {string} message
 * @param {Exception} e 
 */
function command_error(url, message, e) {
    console.error(`Error while fetching ${url} with ${message}`, e)
}

/**
 * Class Command is used between browser and server.
 *
 * Why? We probably change this so better concentrate into one place.
 */
class Command {
    /**
     * @inheritdoc
     *
     * @param {string} cmd
     * @param {string} target
     * @param {string} payload
     */
    constructor(cmd, target, payload) {
        this.cmd = cmd;
        this.target = target;
        this.payload = payload;
    }

    /**
     * Build a base object structure.
     *
     * @returns A command structured object
     */
    to_object() {
        return {
            cmd: this.cmd,
            target: this.target,
            payload: this.payload
        };
    }
}

function findIDs(target) {
    let children = target.children;
    if (children.length !== 1) {
        console.error(`The content MUST HAVE a wrapper!!! Found ${children.length} elements.`);
        return '';
    }
    else {
        const id = children[0].id;
        if (id) {
            console.log(`ID '${id}' found`);
            return id;
        }
        else {
            console.error(`NO ID found in ${target.innerHTML.substring(0,100)}`);
            return ''
        }
    }
}

function apply_command(cmd) {
    const stamp = new Date().toISOString().split('T')[1];
    let con = document.getElementById('console');
    if (con) { //}) && !cmd.target == 'console') {
        con.innerHTML += `<li>${stamp} ${cmd.cmd}: ${cmd.target}</li>`;
    }
    const target = document.getElementById(cmd.target);
    if (target) {
        // (clear, element-parent-id, '')   clear a list of items ie console log
        // (navigate, iframe-id, url)       navigate the iframe to something differently
        // (replace, element-id, 'content') replace the content of an element
        if (cmd.target !== 'console') {
            console.debug(`Trying to '${cmd.cmd}' on target '${cmd.target}'`);
        }
        let success = true;
        // Use target functions to add/clear/remove data
        // Make sure to get new content (newElement) added to use for
        // var elements = newElement.getElementsByClassName(names);
        // document.removeChild(document.documentElement);

        // When adding/replacing HTML is MUST be a wrapped item.
        // When needing to add multiple elements use the result LIST!!
        let elementID = '';
        let element = '';
        switch (cmd.cmd) {
            case 'append':
            case 'replace':
                element = document.createElement('div');
                element.innerHTML = cmd.payload;
                if (cmd.target !== 'console') {
                    elementID = findIDs(element);
                }
                break;
        }

        switch (cmd.cmd) {
            case 'append':
              target.innerHTML += cmd.payload;
              break;
            case 'clear':
              target.innerHTML = '';
              break;
            case 'replace':
              target.innerHTML = cmd.payload;
              break;
            case 'navigate' && target.tagName.toLowerCase() == 'iframe':
              target.src = cmd.payload;
              break;

            default:
              success = false;
              console.error(`Not yet implemented cmd: ${cmd.cmd}.`);
              if (con && !cmd.target == 'console') {
                con.innerHTML += `<li>${stamp} Not implemented cmd: ${cmd.cmd} or invalid ${target.tagName}.</li>`;
              }
        }
        if (success) {
            console.log("XXXX ID", elementID)
            if (elementID) {
                activateElement(elementID);
            }
            
        }
    }
    else {
        console.error(`Cannot find target ${cmd.target}`);
    }
}

function get_commands() {
    let path = '/api/commands';
    console.debug("Fetching", path);
    fetch(path)
        .then(response => {
            if (!response.ok) {
                throw new Error("HTTP error " + response.status);
            }
            return response.json();
        })
        .then(json => {
            console.debug("JSON", json);
            // [{ cmd: "A", target: "B", payload: "C" }]

            // debugger;
            
            json.forEach((cmd, i, a) => {
                console.debug(i, cmd);
                apply_command(cmd);
            });
        })
        .catch(function () {
            console.error("Unable to fetch", )
            this.dataError = true;
        })
}

function send_command(data){
    // data = { cmd: "A", target: "B", payload: "C" };
    console.log('Sending', data);
    fetch("/api/add-command", {
        method: "POST",
        headers: {'Content-Type': 'application/json'}, 
        body: JSON.stringify(data)
    }).then(res => {
        console.log("Request complete! response:", res);
    });
}

/**
 * Send form elements as command.
 *
 * Used by web/components/debug-controller.html
 */
function send_form_command() {
    const cmd = document.getElementById("command-cmd");
    const target = document.getElementById("command-target");
    const payload = document.getElementById("command-payload");
    if (cmd && target && payload) {
        const data = { cmd: cmd.value, target: target.value, payload: payload.value};
        console.log(data);
        send_command(data);
    }
}

/**
 * Sends a special command to server which takes care of cmd and target.
 *
 * @param {string} payload value from a HTML commands attribute.
 */
function get_server_response(payload) {
    get_static((new Command('server', 'payload', payload)).to_object());
}

/**
 * Requests HTML content based on command.
 *
 * @param {Command} cmd action to take.
 */
function get_static(cmd) {
    console.log('Sending', cmd);
    fetch("/fetch", {
        method: "POST",
        headers: {'Content-Type': 'application/json'},
        body: JSON.stringify(cmd)
    }).then(response => {
        console.debug(response);
        return response.json();
    }).then(json => {
        if (!Array.isArray(json)) {
            json = [json];
        }
        json.forEach(cmd => apply_command(cmd));
    });
}

/** run every X millisceonds */
// let pollerID = setInterval(get_commands, 5000);

/**
 * This is bad code and should be replaced by context attach eventhandlers
 */
function activateAll() {
    activateTabs();
    activateTrees();
	activateMaps();
}

function activateElement(id) {
    let element = document.getElementById(id);
    if (element && element.id === id) {
        activateTrees(element);
        activateMaps(element);
        activateTabs(element)
    }
    else {
        console.error(`Cannot find element with ID ${id}`);
    }
}
function runTests() {
    let cmds = [
        new Command('replace', 'main', '<h2>Testing</h2>'),
        new Command('append', 'main','<iframe src=/tests/controller-test.html></iframe>'),
    ];
    cmds.forEach(cmd => apply_command(cmd))

    let payloads = [
        '/data/grip_broser.html',
        '/data/grip_browse1.html',
        '/data/grip_compound.html',
        '/data/grip_frame.html',
        '/data/grip_menu.html',
        '/data/grip_org.html',
        '/data/agrip_pie.html',
        '/data/grip_tree.html',
        '/data/grip_wordtree.html',
        '/data/grip_world.html',
    ];
    payloads.forEach(payload => get_static(new Command('append', 'main', payload)));

    let weerwolven = [
        '| LABEL [MAIN] H_TABS   | CALL MENU_BEHEER   |',
        '| |BROWSE select id, source , target  from xxx.grip_aud | WHERE 1=1 | |',
        '| LABEL [MAIN] H_TABS   | CALL MENU_TABLE| OBJECT GRIP_LOG |',
        // Hieronder foute commands uit grip_menu.html ... je print met " EN je WHERE gebruikt dat.
        // Zie ook dubbel click data/grip_menu.html opent browser ... dan view source ... rood in firefox is fout
        '| |BROWSE select id, source , target  from xxx.grip_metadata_v | WHERE data_type = " TABLE" | |'
    ];
}

/**
 * Add 'active' class on item and remove from others.
 *
 * The item clicked belongs to a group (<ul.tree>, <div.buttons>)
 * and all items get removed the class 'active' and given item
 * or its nearest parent (<li>, <button>) get this.
 *
 * @param {*} item item clicks on
 */
function makeItemActivate(item) {
    console.log('activateItem', item);
    let list = [];
    let me = item;

    let parent = item.closest('ul.tree')
    if (parent) {
        list = document.querySelectorAll('li', parent);
        // item is <a> so take its <li>
        me = item.closest('li');
    }
    else {
        parent = item.closest('div.buttons')
        if (parent) {
            list = document.querySelectorAll('button', parent);
        }
    }

    if (parent) {
        console.log('activateItem.parent', parent);
        for(var i = 0; i < list.length; i++) {
            list[i].classList.remove('active');
        }
        me.classList.add('active')
    }
}

function call_grip(item) {
    console.log(item);
    if (item.hasAttribute('commands')) {
        makeItemActivate(item);
        const commands = item.getAttribute('commands')
        console.log('tree commands', commands);
        get_server_response(commands);
    }
    return false;
}

function post_form_grip(form) {
    // let formData = new FormData(form);
    // console.log(form, formData);
    const formID = form.getAttribute('id');
    let payload = {
        form: formID,
        data: {}
    };
    for(let i= 0 ; i < form.length; i++) {
        const field = form[i];
        const fieldType = field.tagName;
        const dataId = field.getAttribute('name');
        const inputType = field.getAttribute('type');
        let  dataValue = field.value;
        console.log(i, fieldType, dataId, dataValue,inputType );
		if ( inputType == 'checkbox' ) {
		    //console.log( field, field.checked );
			if ( !field.checked ) dataValue = '';
		}
		if ( inputType == 'radio' ) {
		    //console.log( field, field.checked );
			if ( !field.checked ) continue; 
		}

        payload.data[dataId] = dataValue;
    }
    console.log(payload);

    get_static((new Command('form', 'payload', JSON.stringify(payload))).to_object());
    // prevent navigation
    return false;
}


/**
 * Hieronder de routines van JAGGER
 */
function CheckBoxArray( pid ) {
	const nodeList = document.querySelectorAll(".CheckBoxArray");  // scan alle class=array gevallen. 
	for (let i = 0; i < nodeList.length; i++) {
		//alert("I am an alert box!   " +  nodeList[i].checked  );
		nodeList[i].checked = false
		if ( nodeList[i].id == pid ) {
			nodeList[i].checked = true
			if ( nodeList[i].value != '' ) {   // commando zit in checkbox.value , deze uitvoeren !
				get_server_response( nodeList[i].value )  
			}
		}				
	}
}
		